/************************************************************************
* EPP RTK C++
* Copyright (C) 2001 Global Name Registry 
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact information:  epprtk@gnr.com
*
*                       EPP RTK
*                       GNR Ltd.
*                       125 High Holborn
*                       London WC1V 6QA
*                       United Kingdom
************************************************************************/

#include <vector>
#include <iostream>
#ifdef USELIBIDN
#include <idna.h>
#endif

#include "epp-rtk-cpp/epp_DomainCheck.h"
#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"

#include "IDN.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;

void printRsp(const epp_DomainCheckRsp_ref req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      printResponsePart(req->m_rsp);
      
      if(req->m_results != NULL) {
	 for(epp_check_result_seq::iterator i =req->m_results->begin(); i != req->m_results->end(); i++)
	   {
	      string exists = *i->m_exists ? "Y" : "N";
	      cout << "m_exists: " << exists << endl;
	      cout << "m_value: " << *i->m_value << endl;
	   }
      }
   }
   
   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {
   
   epp_Session test;
   
   string username, password;
   
   if(!handleArgs(test, argc, argv, username, password))
     return 1;
   
   testLogIn(test, username, password);
   
   try {
      
      epp_DomainCheckReq_ref l_req(new epp_DomainCheckReq());
      
      IDN_ref idn(new IDN());
      idn->m_command.ref(new epp_string("check"));
      idn->m_script.ref(new epp_string("de"));
      
      l_req->m_cmd.ref(new epp_Command(NULL,
				       idn,
				       epp_trid("ABC-12346")));
      
      epp_string_seq_ref names(new epp_string_seq());
      
#ifdef USELIBIDN
      char *puny_domain_name;
      int puny_error;

      uint32_t unicode_domain_name1[] = { 0x00e4,
                                          0x002e,
                                          0x0069,
                                          0x006e,
                                          0x0066,
                                          0x006f,
                                          0x0 }; // .info
      puny_error = idna_to_ascii_4z( unicode_domain_name1, &puny_domain_name, IDNA_USE_STD3_ASCII_RULES );
      if ( puny_error ) {
          cout << "Failed to punycode the domain name ["<<puny_error<<"]" << endl;
      } else {
          cout << "First punycode domain name is: ["<<puny_domain_name<<"]" << endl;
          names->push_back(puny_domain_name);  
      }
      free(puny_domain_name);  // must use "free" and not "delete", because the punycoder is C-based

      uint32_t unicode_domain_name2[] = { 0x00f6,
                                          0x002e,
                                          0x0069,
                                          0x006e,
                                          0x0066,
                                          0x006f,
                                          0x0 }; // .info
      puny_error = idna_to_ascii_4z( unicode_domain_name2, &puny_domain_name, IDNA_USE_STD3_ASCII_RULES );
      if ( puny_error ) {
          cout << "Failed to punycode the domain name ["<<puny_error<<"]" << endl;
      } else {
          cout << "Second punycode domain name is: ["<<puny_domain_name<<"]" << endl;
          names->push_back(puny_domain_name);  
      }
      free(puny_domain_name);  // must use "free" and not "delete", because the punycoder is C-based

      uint32_t unicode_domain_name3[] = { 0x00fc,
                                          0x002e,
                                          0x0069,
                                          0x006e,
                                          0x0066,
                                          0x006f,
                                          0x0 }; // .info
      puny_error = idna_to_ascii_4z( unicode_domain_name3, &puny_domain_name, IDNA_USE_STD3_ASCII_RULES );
      if ( puny_error ) {
          cout << "Failed to punycode the domain name ["<<puny_error<<"]" << endl;
      } else {
          cout << "Third punycode domain name is: ["<<puny_domain_name<<"]" << endl;
          names->push_back(puny_domain_name);  
      }
      free(puny_domain_name);  // must use "free" and not "delete", because the punycoder is C-based
#else

      // No libidn available, so let's just put in the punycode strings by hand.

      cout << "Using pre-punied domain names..." << endl;
      names->push_back("xn--4ca.info");  // .info
      names->push_back("xn--nda.info");  // .info
      names->push_back("xn--tda.info");  // .info

#endif

      l_req->m_names = names;
      
      epp_DomainCheck_ref l_obj(new epp_DomainCheck());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_DomainCheckRsp_ref l_rsp = l_obj->getResponseData();
      
      printRsp(l_rsp);
      
   }
   catch(epp_XMLException ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(epp_Exception ex) {
      cout << "Exception: " << endl;
      printResultsSeq(ex.getDetails());
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }

}
