/************************************************************************
* EPP RTK C++
* Copyright (C) 2001 Global Name Registry 
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact information:  epprtk@gnr.com
*
*                       EPP RTK
*                       GNR Ltd.
*                       125 High Holborn
*                       London WC1V 6QA
*                       United Kingdom
************************************************************************/

#include <vector>
#include <iostream>

#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_DomainInfo.h"
#include "epp-rtk-cpp/epp_DomainCreate.h"
#include "epp-rtk-cpp/epp_DomainDelete.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"

#include "DomainTrademark.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;

void printRsp(const epp_DomainCreateRsp_ref req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      
      printResponsePart(req->m_rsp);
      
      if(req->m_name != NULL)
	cout << "m_name: " << *req-> m_name << endl;
      if(req->m_roid != NULL)
	cout << "m_roid: " << *req-> m_roid << endl;
      if(req->m_expiration_date != NULL)
	cout << "m_expiration_date: " << *req-> m_expiration_date << endl;
   }
   
   
   cout << "_______________________________\n";
} // printRsp

void printRsp(const epp_DomainInfoRsp_ref req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      
      printResponsePart(req->m_rsp);
      
      if(req->m_name != NULL)
	cout << "m_name: " << *req->m_name << endl;
      
      if(req->m_roid != NULL)
	cout << "m_roid: " << *req->m_roid << endl;
      
      if(req->m_status != NULL)
	{
	   cout << "m_status:\n";
	   for(epp_domain_status_seq::iterator i = req->m_status->begin();
	       i != req->m_status->end(); i++)
	     {
		if(i->m_type != NULL) {
		   string status;
		   switch(*i->m_type) {
		    case HOLD: status = "HOLD"; break;
		    case INACTIVE: status = "INACTIVE"; break;
		    case LOCK: status = "LOCK"; break;
		    case NEW: status = "NEW"; break;
		    case OK: status = "OK"; break;
		    case PENDING_DELETE: status = "PENDING_DELETE"; break;
		    case PENDING_TRANSFER: status = "PENDING_TRANSFER"; break;
		    case PENDING_VERIFICATION: status = "PENDING_VERIFICATION"; break;
		    case CLIENT_HOLD: status = "CLIENT_HOLD"; break;
		    case CLIENT_LOCK: status = "CLIENT_LOCK"; break;
		    case CLIENT_TRANSFER_PROHIBITED: status = "CLIENT_TRANSFER_PROHIBITED"; break;
		    default: status = "**UNKNOWN**";
		   }
		   cout << " m_type: " << status << endl;
		   
		}
		if(i->m_lang != NULL)
		  cout << " m_lang :" << *i->m_lang << endl;
		if(i->m_value != NULL)
		  cout << " m_value: " << *i->m_value << endl;
	     }
	}
      
      if(req->m_registrant != NULL)
	cout << "m_registrant: " << *req->m_registrant << endl;
      
      if(req->m_contacts != NULL)
	{
	   cout << "m_contacts:\n";
	   for(epp_domain_contact_seq::iterator i = req->m_contacts->begin();
	       i != req->m_contacts->end(); i++)
	     {
		if(i->m_type != NULL) {
		   string type;
		   switch(*i->m_type) {
		    case ADMIN: type = "ADMIN"; break;
		    case BILLING: type = "BILLING"; break;
		    case TECH: type = "TECH"; break;
		    default: type = "**UNKNOWN**";
		   }
		   cout << " m_type: " << type << endl;
		   
		}
		if(i->m_roid != NULL)
		  cout << " m_roid: " << *i->m_roid << endl;
	     }
	}
      
      if(req->m_name_servers != NULL)
	{
	   cout << "m_name_servers:\n";
	   for(epp_string_seq::iterator i = req->m_name_servers->begin();
	       i != req->m_name_servers->end(); i++)
	     {
		  cout << " ns: " << *i << endl;
	     }
	}
      
      if(req->m_hosts != NULL)
	{
	   cout << "m_hosts:\n";
	   for(epp_string_seq::iterator i = req->m_hosts->begin();
	       i != req->m_hosts->end(); i++)
	     {
		  cout << " host: " << *i << endl;
	     }
	}
      
      if(req->m_client_id != NULL)
	cout << "m_client_id: " << *req->m_client_id << endl;
      
      if(req->m_created_by != NULL)
	cout << "m_created_by: " << *req->m_created_by << endl;
      
      if(req->m_created_date != NULL)
	cout << "m_created_date: " << *req->m_created_date << endl;
      
      if(req->m_updated_by != NULL)
	cout << "m_updated_by: " << *req->m_updated_by << endl;
      
      if(req->m_updated_date != NULL)
	cout << "m_updated_date: " << *req->m_updated_date << endl;
      
      if(req->m_expiration_date != NULL)
	cout << "m_expiration_date: " << *req-> m_expiration_date << endl;
      
      if(req->m_transfer_date != NULL)
	cout << "m_transfer_date: " << *req->m_transfer_date << endl;
      
      if(req->m_auth_info != NULL) {
	 cout << "m_auth_info: " << endl;
	 
	 if(req->m_auth_info->m_type != NULL) {
	    string type;
	    switch(*req->m_auth_info->m_type) {
	     case PW: type = "PW"; break;
	     default: type = "**UNKNOWN**";
	    }
	    cout << " m_type: " << type << endl;	    
	 }
	 
	 if(req->m_auth_info->m_roid != NULL)
	   cout << " m_roid: " << *req->m_auth_info->m_roid << endl;

	 if(req->m_auth_info->m_value != NULL)
	   cout << " m_value: " << *req->m_auth_info->m_value << endl;
      }
      
   } // if(req != NULL)
   
   cout << "_______________________________\n";
} // printRsp


int main(int argc, char * argv[]) {
   
   epp_Session test;
   
   string username, password;
   
   if(!handleArgs(test, argc, argv, username, password))
     return 1;
   
   testLogIn(test, username, password);
   
   try {
       
      epp_DomainCreateReq_ref l_req(new epp_DomainCreateReq());
      
      DomainTrademark_ref trademark(new DomainTrademark());
      
      trademark->m_name.ref(new epp_string("ACME Kitchen Products"));
      trademark->m_number.ref(new epp_string("016-4C5V-339A"));
      trademark->m_country.ref(new epp_string("European Union"));
      trademark->m_date.ref(new epp_string("Jan 1, 1998"));
      
      l_req->m_cmd.ref(new epp_Command(NULL,
				       trademark,
				       epp_trid("ABC-12345")));
      
      l_req->m_name.ref(new epp_string("acme.info"));
      l_req->m_period.ref(new epp_DomainPeriod(YEAR, 2));
      
      l_req->m_auth_info.ref(new epp_AuthInfo());
      l_req->m_auth_info->m_type.ref(new epp_AuthInfoType(PW));
      l_req->m_auth_info->m_value.ref(new epp_string("2fooBAR"));
      
      epp_DomainCreate_ref l_obj(new epp_DomainCreate());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_DomainCreateRsp_ref l_rsp = l_obj->getResponseData();
      
      printRsp(l_rsp);
      
      return 0;
      
    }
    catch(epp_XMLException ex) {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(epp_Exception ex) {
        cout << "Exception: " << endl;
        printResultsSeq(ex.getDetails());
    }
    catch(...) {
        cout << "Something else went wrong...\n";
    }
   
   try {
      
      epp_DomainInfoReq_ref l_req(new epp_DomainInfoReq());

      l_req->m_cmd.ref(new epp_Command(NULL,
				       epp_Unspec_ref(new epp_Unspec()),
				       epp_trid("ABC-12346")));
      
      l_req->m_name.ref(new epp_string("acme.info"));
      
      epp_DomainInfo_ref l_obj(new epp_DomainInfo());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_DomainInfoRsp_ref l_rsp = l_obj->getResponseData();
      
      printRsp(l_rsp);

      // Let us do something more with the unspec-object:
      if(l_rsp->m_rsp->m_unspec_string != NULL)
	{
	    DomainTrademark_ref trademark(new DomainTrademark());
	    trademark->fromXML(epp_string(*l_rsp->m_rsp->m_unspec_string));
            cout << "Trademark name: " << *trademark->m_name << endl;
            cout << "Trademark number: " << *trademark->m_number << endl;
            cout << "Trademark country: " << *trademark->m_country << endl;
            cout << "Trademark date: " << *trademark->m_date << endl;
		
	}
      
      
      return 0;
      
   }
   catch(epp_XMLException ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(epp_Exception ex) {
      cout << "Exception: " << endl;
      printResultsSeq(ex.getDetails());
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }

   try {
      
      epp_DomainDeleteReq_ref l_req(new epp_DomainDeleteReq());

      l_req->m_cmd.ref(new epp_Command(NULL,
				       epp_Unspec_ref(new epp_Unspec()),
				       epp_trid("ABC-12346")));
      
      l_req->m_name.ref(new epp_string("acme.info"));
      
      epp_DomainDelete_ref l_obj(new epp_DomainDelete());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_DomainDeleteRsp_ref l_rsp = l_obj->getResponseData();
      
      return 0;
      
   }
   catch(epp_XMLException ex) {
      cout << "Exception: " << ex.getString() << endl;
   }
   catch(epp_Exception ex) {
      cout << "Exception: " << endl;
      printResultsSeq(ex.getDetails());
   }
   catch(...) {
      cout << "Something else went wrong...\n";
   }

}
