/************************************************************************
* EPP RTK C++
* Copyright (C) 2001 Global Name Registry 
* 
* This library is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as published
* by the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
*
* This library is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
* Contact information:  epprtk@gnr.com
*
*                       EPP RTK
*                       GNR Ltd.
*                       125 High Holborn
*                       London WC1V 6QA
*                       United Kingdom
************************************************************************/

#include <vector>
#include <iostream>

#include "epp-rtk-cpp/data/epp_XMLException.h"
#include "epp-rtk-cpp/data/epp_Exception.h"
#include "epp-rtk-cpp/epp_Action.h"
#include "epp-rtk-cpp/epp_DomainRenew.h"
#include "epp-rtk-cpp/epp_Session.h"
#include "epp-rtk-cpp/testTools.h"

#include "RGPRenew.h"

using namespace std;
using namespace eppobject::domain;
using namespace eppobject::epp;

void printRsp(const epp_DomainRenewRsp_ref req)
{
   cout << "Printing Response:\n_______________________________\n";
   
   if(req != NULL) {
      
      printResponsePart(req->m_rsp);
      
      if(req->m_name != NULL)
	cout << "m_name: " << *req-> m_name << endl;
      if(req->m_expiration_date != NULL)
	cout << "m_expiration_date: " << *req-> m_expiration_date << endl;
   }
   
   
   cout << "_______________________________\n";
} // printRsp

int main(int argc, char * argv[]) {
   
   epp_Session test;
   
   string username, password;
   
   if(!handleArgs(test, argc, argv, username, password))
     return 1;
   
   testLogIn(test, username, password);
   
   try {
       
      epp_DomainRenewReq_ref l_req(new epp_DomainRenewReq());
      
      RGPRenew_ref rgprenew(new RGPRenew());
      
      l_req->m_cmd.ref(new epp_Command(NULL,
				       rgprenew,
				       epp_trid("ABC-12345")));
      
      l_req->m_name.ref(new epp_string("valid.info"));
      l_req->m_current_expiration_date.ref(new epp_date("2000-04-03"));
      
      epp_DomainRenew_ref l_obj(new epp_DomainRenew());
      
      l_obj->setRequestData(*l_req);
      
      test.processAction(l_obj);
      
      epp_DomainRenewRsp_ref l_rsp = l_obj->getResponseData();
      
      printRsp(l_rsp);
      
      return 0;
      
    }
    catch(epp_XMLException ex) {
        cout << "Exception: " << ex.getString() << endl;
    }
    catch(epp_Exception ex) {
        cout << "Exception: " << endl;
        printResultsSeq(ex.getDetails());
    }
    catch(...) {
        cout << "Something else went wrong...\n";
    }
}
